﻿(function () {
	"use strict";

	angular.module("starter.services")
	.service("multimediaService", function ($cordovaCamera, $q, userService, apiService, $rootScope) {
			var self =  this

			this.getImageFromGallery = function () {
				var deffered = $q.defer();
				var galleryOptions = {
					quality: 100,
					// targetWidth: 200,
					// targetHeight: 200,
					allowEdit: false,
					destinationType: Camera.DestinationType.FILE_URI,
					sourceType: Camera.PictureSourceType.PHOTOLIBRARY,
					mediaType: Camera.MediaType.ALLMEDIA
				};
				$cordovaCamera.getPicture(galleryOptions).then(url => {
					let extIndex = url.indexOf('com.android.externalstorage.documents/document/')
					if (extIndex != -1) {
						url = 'file:///storage/' + decodeURIComponent(url.replace('%3A', '%2F').substr(extIndex + 47))
					}
					extIndex = url.indexOf('/storage/emulated/')
					if (extIndex == 0)
						url = 'file://' + url

		  		window.FilePath.resolveNativePath(url, finalPath => {
						deffered.resolve(finalPath);
					}, err => {
						deffered.reject(err);
					})
				}, err => {
					deffered.reject(err);
				})
				return deffered.promise;
			}

			/** */
			this.openCamera = function () {
				var cameraOptions = {
					quality: 100,
					destinationType: Camera.DestinationType.DATA_URL,
					sourceType: Camera.PictureSourceType.CAMERA,
					allowEdit: false,
					encodingType: Camera.EncodingType.JPEG,
					// targetWidth: 200,
					// targetHeight: 200,
					popoverOptions: CameraPopoverOptions,
					correctOrientation: true,
					saveToPhotoAlbum: false
				};
				//var opt = Object.assign({}, cameraOptions); //Cloning the cameraOptions object
				var opt = angular.copy(cameraOptions); //Cloning the cameraOptions object
				opt.destinationType = Camera.DestinationType.FILE_URI;
				return $cordovaCamera.getPicture(opt);
			}

			this.uploadPicture = function(uri, ftype, ownerid = '1', ownertable = 'tstore') {
				var deffered = $q.defer();
				var fileName = uri.substr(uri.lastIndexOf('/')+1);
				fileName = fileName.indexOf('?') != -1 ? fileName.substr(0, fileName.indexOf('?')) : fileName
				self.getOBSURL(fileName, ftype, ownerid, ownertable)
				.then(res => {
					if (res.data.code == 1) {
						self.uploadOBS(uri, res.data.data.obsuploadurl, res.data.data.obsfilename)
						.then(res => {
							deffered.resolve(res);
							// console.log(res)
						}, err => {
							deffered.reject(err);
							// console.log(err)
						})
					} else
						deffered.reject(res.data.data);

				})
				
				return deffered.promise;
			}

			this.writeStream = function(fileWriter, data) {
				return new Promise((resolve, reject) => {
					let written = 0;
					let BLOCK_SIZE = 1024 * 1024 * 4; // write 4MB every time of write
					async.whilst(
						cb => {
							cb(null, written < data.size)
						}, cb => {
							let sz = Math.min(BLOCK_SIZE, data.size - written)
							let sub = data.slice(written, written + sz)
							written += sz
							try {
								fileWriter.write(sub)
							} catch(err) {
								reject(err)
							}
							fileWriter.onwrite = function(evt) {
								cb(null)
							}
						}, err => {
							if (err)
								reject(err)
							else
								resolve(true)
					})
				})
			}

			this.downloadFile = function(uri) {
				var deffered = $q.defer();
				if (!uri) {
					deffered.reject(false)
					return deffered.promise;
				}
						axios({
							url: decodeURI(uri),
							method: 'get',
							withCredentials: false, 
							validateStatus: function(status){
								return status >= 200;
							},
							maxRedirects : 0,
							responseType: 'blob', // important
							onDownloadProgress (progress) {
								$rootScope.round_progress = Math.round(progress.loaded / progress.total * 100)
							}
						}).then(axiosResponse => {
							var filename = uri.substr(uri.lastIndexOf('/')+1)
							filename = filename.indexOf('?') != -1 ? filename.substr(0, filename.indexOf('?')) : filename
							let ext = filename.substring(filename.lastIndexOf('.') + 1)
							let DOWNLOAD_DIR = cordova.file.externalRootDirectory + "/Download"
							if (ext.toUpperCase() == 'PNG' || ext.toUpperCase() == 'JPG' || ext.toUpperCase() == 'JPEG' || ext.toUpperCase() == 'BMP' || ext.toUpperCase() == 'GIF' 
									|| ext.toUpperCase() == 'MP4' || ext.toUpperCase() == 'MOV' || ext.toUpperCase() == 'AVI' || ext.toUpperCase() == '3GP')
								DOWNLOAD_DIR = cordova.file.externalCacheDirectory
							window.resolveLocalFileSystemURL(DOWNLOAD_DIR, entry => {
								entry.getFile(filename, { create: true, exclusive: false}, function (fileEntry) {
									fileEntry.createWriter(function (fileWriter) {
										// fileWriter.onwriteend = function() {
										// 	deffered.resolve(fileEntry);
										// };
										// fileWriter.write(new Blob([axiosResponse.data]));
										fileWriter.onerror = function (e) {
											deffered.reject(err);
										};

										self.writeStream(fileWriter, new Blob([axiosResponse.data]))
										.then(res => deffered.resolve(fileEntry))
										.catch(err => deffered.reject(err))				
									});
								});
							}, err => {
								deffered.reject(err);
							});
						}).catch(function (err) {
							deffered.reject(err);
						});
				
				return deffered.promise;
			}
			
			this.getImageList = function(ownerid, ownertable = 'tstore') {
				let request = {
					url: API_URL_V1,
					data: {
						"classname": "system.system.docManage",
						"method": "queryDoc",
						"content": {
							"userid": userService.getUserId(),
							"getdatafromdbanyway": true,
							"ownertable": ownertable,
							"ownerid": ownerid
						}
					}
				};

				return apiService.makeRequest(request);
			}

			this.getOBSURL = function(fileName, ftype, ownerid = '1', ownertable = 'tstore') {
				var filetype = fileName.substr(fileName.lastIndexOf('.')+1);
				let request = {
					url: API_URL_V1,
					data: {
						"classname": "system.system.docManage",
						"method": "getFileName",
						"content": {
							"userid": userService.getUserId(),
							"filename": fileName,
							"filetype": filetype,
							"ownertable": ownertable,
							"ownerid": ownerid,
							"ftype": ftype
						}
					}
				};
	
				return apiService.makeRequest(request);
			};

			this.removeImage = function(item) {
				let request = {
					url: API_URL_V1,
					data: {
						"classname": "system.system.docManage",
						"method": "deleteDoc",
						"content": {
							"userid": userService.getUserId(),
							"ownertable": item.ownertable,
							"ownerid": item.ownerid,
							"tattachmentid": item.tattachmentid
						}
					}
				};

				return apiService.makeRequest(request);
			}

			this.uploadOBS = function(fileURI, obsURL, obsfilename) {
				var deffered = $q.defer();
				var fileName = obsURL.substr(obsURL.lastIndexOf('/')+1);
				fileName = fileName.indexOf('?') != -1 ? fileName.substr(0, fileName.indexOf('?')) : fileName
				window.resolveLocalFileSystemURL(fileURI, fileEntry => {
					fileEntry.file(function (file) {
						var reader = new FileReader();
						reader.onloadend = function() {
							var blob = new Blob([new Uint8Array(this.result)]);
							blob.name = fileName
							var reopt = {
								method : 'PUT',
								url : obsURL,
								timeout: 3600000,
								withCredentials: false, 
								headers : {'Content-Type' : 'application/octet-stream'},
								validateStatus: function(status){
										return status >= 200;
								},
								maxRedirects : 0,
								responseType : 'text',
								onUploadProgress: function (e) {
									var percent = (e.loaded / e.total)*100
									// e.loaded / e.total === 1?that.show = false:''
								},
								data : blob,
								// data:'1'
							};
							// 华为云上传
							axios.request(reopt).then((res)=>{
								self.uploadSuccess(obsfilename, 1).then(res => {
									deffered.resolve(res);
								})
							}, err => {
								deffered.reject(res);
							})
						};
						reader.readAsArrayBuffer(file);
					});
				})

				return deffered.promise;
			}

			this.uploadSuccess = function(obsfilename, fisreturnallfile = "1") {
				let request = {
					url: API_URL_V1,
					data: {
						"classname": "system.system.docManage",
						"method": "uploadSuccsess",
						"content": {
							"userid": userService.getUserId(),
							"obsfilename": obsfilename,
							"fisreturnallfile": fisreturnallfile
						}
					}
				};
	
				return apiService.makeRequest(request);
			}

		});

})();