(function(){
	"use strict";

	angular.module('starter.controllers')
    .controller('ShijianFenxiXitongShouyeController', function($rootScope, $scope, $ionicPopup, userService, fenxiService, qiyeShujuService) {
      $scope.showHeader();
      $scope.hideTabBar();

      $scope.chartOption = 0;
      $scope.changeChartOption = () => {
        $scope.chartOption = 1 - $scope.chartOption;
        $scope.loadData();
      }

      let chart = new G2.Chart({
        container: 'shijianfenxi-chart',
        autoFit: true,
        height: 250,
      });
      $scope.$on('$destroy', function() {
        chart.clear()
        if (!chart.destroyed)
          chart.destroy()
      });
      
      $scope.initFenxiPeriod();

      $scope.loadData = () => {
        if (!$scope.fenxiPeriodStart) {
          $scope.setFenxiPeriodStart(new Date());
          $scope.setFenxiPeriodEnd(new Date());
          $scope.setFenxiPeriodString($scope.fenxiPeriodStart.getFullYear() + '年' + ("0" + ($scope.fenxiPeriodStart.getMonth() + 1)).slice(-2) + '月');
        }

        $scope.hideChart1 = false;
        (userService.getRole() == '企业' ? qiyeShujuService : fenxiService).queryEventOfOrders($scope.fenxiPeriodStart, $scope.fenxiPeriodEnd)
        .then(function(res) {
          if (res.data.code === 1) {
            if (res.data.data.length > 0 && res.data.data[0].fsaleevent) {
              $scope.tips = isRangeDate($scope.fenxiPeriodStart, $scope.fenxiPeriodEnd) ? res.data.tips : '';

              var data = [];

              var sum = 0;
              res.data.data.forEach(function(v) {
                if (v.fcount != 0) {
                  sum += ($scope.chartOption) == 0 ? v.fcount : v.forderamount;
                  data.push(v);
                }
              });

              if (sum == 0) {
                $scope.hideChart1 = true;
              }
              chart.clear();
              chart.data(data);
              if ($scope.chartOption == 0) {
                chart.scale('fcount', {
                  formatter: function(val){
                    val = val + '件';
                    return val;
                  },
                });
              } else {
                chart.scale('forderamount', {
                  formatter: function(val){
                    if (val >= 1000) {
                      val = parseFloat((val / 1000).toFixed(2)) + 'K';
                    }
                    return '￥' + val;
                  },
                });
              }
              chart.coordinate('theta', {
                radius: 0.7,
                // innerRadius: 0.3,
              });
              
              if ($scope.chartOption == 0) {
                chart
                .interval()
                .adjust('stack')
                .position('fcount')
                .color('fsaleevent')
                .label('fcount', {
                  layout: [{ type: 'pie-spider' }, { type: 'hide-overlap' }],
                  offset: 8,
                  content: function(obj) {
                    const G = G2.getEngine('canvas');
                    const group = new G.Group({});
                    const [y0, y1] = [0, 0];
                    const inRight = y0 < y1;
                    const textAlign = inRight ? 'left' : 'right';
                    const topFontSize = 12;
                    group.addShape({
                      type: 'text',
                      attrs: {
                        x: 0,
                        y: 0,
                        text: `${obj.fsaleevent}\n${obj.fcount}件\n${parseFloat((obj.fcount / sum * 100).toFixed(2))}%`,
                        fill: 'rgba(0, 0, 0, 0.65)',
                        fontSize: topFontSize,
                        textAlign,
                      },
                    });
                    if (!inRight) {
                      group.translate(group.getBBox().width, 0);
                    }
                    group.translate(0, topFontSize);
                    return group;
                  },
                  labelLine: {
                    style: {
                      lineWidth: 0.5,
                    },
                  },
                })
              } else {
                chart
                .interval()
                .adjust('stack')
                .position('forderamount')
                .color('fsaleevent')
                .label('forderamount', {
                  layout: [{ type: 'pie-spider' }, { type: 'hide-overlap' }],
                  offset: 8,
                  content: function(obj) {
                    const G = G2.getEngine('canvas');
                    const group = new G.Group({});
                    const [y0, y1] = [0, 0];
                    const inRight = y0 < y1;
                    const textAlign = inRight ? 'left' : 'right';
                    const topFontSize = 12;
                    group.addShape({
                      type: 'text',
                      attrs: {
                        x: 0,
                        y: 0,
                        text: obj.forderamount < 1000 ? `${obj.fsaleevent}\n${obj.forderamount}件\n${parseFloat((obj.forderamount / sum * 100).toFixed(2))}%` : `${obj.fsaleevent}\n${(obj.forderamount / 1000).toFixed(2)}K件\n${parseFloat((obj.forderamount / sum * 100).toFixed(2))}%`,
                        fill: 'rgba(0, 0, 0, 0.65)',
                        fontSize: topFontSize,
                        textAlign,
                      },
                    });
                    if (!inRight) {
                      group.translate(group.getBBox().width, 0);
                    }
                    group.translate(0, topFontSize);
                    return group;
                  },
                  labelLine: {
                    style: {
                      lineWidth: 0.5,
                    },
                  },
                })
                //.tooltip(false);
              }

              chart.interaction('element-active');
              chart.legend({flipPage: false});
              chart.render();
            } else {
              $scope.hideChart1 = true;
            }
          } else {
            //$ionicPopup.alert({
            // title: res.data.msg
            // });
            $scope.hideChart1 = true;
          }

          $scope.hideLoading();
        },
        function(err) {
          $scope.hideLoading();
        });
      }

      $scope.showLoading();
      $scope.loadData();

      $scope.itemOnLongPress = function() {
        $scope.downloadImage(chart);
      }

      $scope.$on('refresh', function(ev) {
        $scope.loadData();
      });

      $('.shuju-topmenu-panel').css('height', '0px');
      $('ion-content.shijianfenxixitong').css("top", "44px");

    });
})();

