(function(){
	"use strict";

	angular.module('starter.controllers')
    .controller('XiaoshouFenxiXitongShouyeController', function($rootScope, $scope, $location, $anchorScroll, $ionicPopup, userService, fenxiService, qiyeShujuService, $timeout) {
      $scope.showHeader();
      $scope.hideTabBar();

      $scope.scrollTo = function(target){
        $location.hash(target); // set the location hash
        $anchorScroll();
      };

      $scope.selectTab = function(idx){
        $scope.navIndex = idx;
        $scope.$broadcast('scroll.refreshComplete');
      }

      $scope.showLoading();

      let chartArray = []
      $scope.$on('$destroy', function() {
        chartArray.forEach(chart => {
          chart.clear()
          if (!chart.destroyed)
            chart.destroy()
        })
      });
      
      setTimeout(function() {
        // 销售动作分析
        $scope.hideChart1 = false;
        (userService.getRole() == '企业' ? qiyeShujuService : fenxiService).queryCustomerAction()
        .then(function(res) {
          if (res.data.code === 1 && res.data.data.length == 0) {
            $scope.hideChart1 = true;
          } else if (res.data.code === 1 && res.data.data.length > 0) {
            $scope.tips1 = res.data.tips;
            // Draw charts
            const dv = new DataSet.DataView().source(res.data.data);
            var sum = 0;
            for (var ii = 0; ii < res.data.data.length; ii++){
              if (res.data.data[ii].fcustaction == '进店') {
                sum = res.data.data[ii].fcount;
              }
            }
            if (sum == 0) {
              $scope.hideChart1 = true;
            }
            dv.transform({
              type: 'map',
              callback(row) {
                row.percent = parseFloat((row.fcount / sum * 100).toFixed(2));
                return row;
              },
            });
            const data = dv.rows;
            const chart = new G2.Chart({
              container: 'xiaoshoufenxixitong-shouye-chart-1',
              autoFit: true,
              height: 300,
              padding: [0, 100, 70],
            });
            chartArray.push(chart)
            chart.data(data);
            chart.axis(false);
            chart.tooltip({
              showTitle: false,
              showMarkers: false,
              itemTpl:
                '<li style="margin-bottom:4px;list-style-type:none;padding: 0;">' +
                '<span style="background-color:{color};" class="g2-tooltip-marker"></span>' +
                '{name}<br/>' +
                '<span style="padding-left: 16px;line-height: 16px;">浏览人数：{fcount}</span><br/>' +
                '<span style="padding-left: 16px;line-height: 16px;">占比：{percent}</span><br/>' +
                '</li>',
            });
            chart
              .coordinate('rect')
              .transpose()
              .scale(1, -1);
            chart
              .interval()
              .adjust('symmetric')
              .position('fcustaction*percent')
              .shape('funnel')
              .color('fcustaction')
              .label(
                'fcustaction*fcount',
                (fcustaction, fcount) => {
                  return {
                    content: `${fcustaction} ${fcount}`,
                  };
                },
                {
                  offset: 15,
                  labelLine: {
                    style: {
                      lineWidth: 1,
                      stroke: 'rgba(0, 0, 0, 0.15)',
                    },
                  },
                }
              )
              .tooltip('fcustaction*fcount*percent', (fcustaction, fcount, percent) => {
                return {
                  name: fcustaction,
                  percent: +percent + '%',
                  fcount,
                };
              })
              .animate({
                appear: {
                  animation: 'fade-in'
                },
                update: {
                  annotation: 'fade-in'
                }
              });

            chart.interaction('element-active');
            chart.legend({flipPage: false});
            let pp = 100 / res.data.data.length

            chart.on('beforepaint', () => {
              chart.annotation().clear(true);
              const chartData = chart.getData();
              // 中间标签文本
              chartData.forEach((obj, idx) => {
                let positionY = idx * pp   + pp / 2
                chart.annotation().text({
                  top: true,
                  position: ['50%', `${positionY}%`],
                  content: obj.percent + '%', // 显示的文本内容
                  style: {
                    stroke: null,
                    fill: '#fff',
                    fontSize: 12,
                    textAlign: 'center',
                    shadowBlur: 2,
                    shadowColor: 'rgba(51, 51, 51, .45)',
                  },
                });
              });

              chart.annotation().text({
                top: true,
                content: '* 以进店数作为统计基数',
                position: ['50%', '100%'],
                style: {
                  fill: '#666666',
                  fontSize: 12,
                  fontWeight: 300,
                  textAlign: 'center',
                },
                offsetY: 25,
              });
            });

            chart.render();
          } else {
            //$ionicPopup.alert({
            // title: res.data.msg
            // });
            $scope.hideChart1 = true;
          }
        },
        function(err) {
          $scope.hideChart1 = true;
        });

        // 销售渠道分析
        $scope.hideChart2 = false;
        (userService.getRole() == '企业' ? qiyeShujuService : fenxiService).queryChannelOfOrders()
        .then(function(res) {
          if (res.data.code === 1 && res.data.data.length == 0) {
            $scope.hideChart2 = true;
          } else if (res.data.code === 1 && res.data.data.length > 0) {
            $scope.tips2 = res.data.tips;
            var data = [];

            var sum = 0;
            res.data.data.forEach(function(v) {
              if (v.fcount != 0) {
                sum += v.fcount;
                data.push(v);
              }
            });

            if (sum == 0) {
              $scope.hideChart2 = true;
            }
            let chart = new G2.Chart({
              container: 'xiaoshoufenxixitong-shouye-chart-2',
              autoFit: true,
              height: 300,
            });
            chartArray.push(chart)
            chart.data(data);
            chart.scale('fcount', {
              formatter: function(val) {
                return val + '人';
              },
            });
            chart.coordinate('theta', {
              radius: 0.7
            });

            chart
              .interval()
              .adjust('stack')
              .position('fcount')
              .color('fchannel')
              .label('fcount', {
                layout: [{ type: 'pie-spider' }, { type: 'hide-overlap' }],
                offset: 8,
                content: function(obj) {
                  const G = G2.getEngine('canvas');
                  const group = new G.Group({});
                  const [y0, y1] = [0, 0];
                  const inRight = y0 < y1;
                  const textAlign = inRight ? 'left' : 'right';
                  const topFontSize = 12;
                  group.addShape({
                    type: 'text',
                    attrs: {
                      x: 0,
                      y: 0,
                      text: `${obj.fchannel}\n${obj.fcount}人\n${parseFloat((obj.fcount / sum * 100).toFixed(2))}%`,
                      fill: 'rgba(0, 0, 0, 0.65)',
                      fontSize: topFontSize,
                      textAlign,
                    },
                  });
                  if (!inRight) {
                    group.translate(group.getBBox().width, 0);
                  }
                  group.translate(0, topFontSize);
                  return group;
                },
                labelLine: {
                  style: {
                    lineWidth: 0.5,
                  },
                },
              })

            chart.interaction('element-active');
            chart.legend({flipPage: false});
            chart.render();
          } else {
            //$ionicPopup.alert({
            // title: res.data.msg
            // });
            $scope.hideChart2 = true;
          }
        },
        function(err) {
          $scope.hideChart2 = true;
        });

        // 成交产品分析
        $scope.chartOption1 = 0;
        $scope.changeChartOption1 = () => {
          $scope.chartOption1 = 1 - $scope.chartOption1;
          $scope.queryProductOfOrders();
        }

        const chart = new G2.Chart({
          container: 'xiaoshoufenxixitong-shouye-chart-3',
          autoFit: true,
          height: 300,
        });
        chartArray.push(chart)

        $scope.queryProductOfOrders = function() {
          $scope.hideChart3 = false;
          (userService.getRole() == '企业' ? qiyeShujuService : fenxiService).queryProductOfOrders()
          .then(function(res) {
            if (res.data.code === 1 && res.data.data.length == 0) {
              $scope.hideChart3 = true;
            } else if (res.data.code === 1 && res.data.data.length > 0) {
              const data = res.data.data;
              $scope.tips3 = res.data.tips;

              chart.clear();
              chart.data(data);
              if ($scope.chartOption1 == 0) {
                chart.scale('fqty', {
                  range: [0, 0.9],
                  formatter: function(val){
                    val = val + '件';
                    return val;
                  },
                });
              } else {
                chart.scale('famount', {
                  range: [0, 0.9],
                  formatter: function(val){
                    if (val >= 1000) {
                      val = parseFloat((val / 1000).toFixed(2)) + 'K';
                    }
                    return '￥' + val;
                  },
                });
              }

              chart.tooltip({
                showTitle: false,
                showMarkers: false
              });
              chart.interaction('active-region');
              chart.legend({flipPage: false});

              let colorMaps = [
                '#FF99A6', '#FF6F75', '#FF2641', '#D0021B', '#8E0909', '#6B0004'
              ];

              if ($scope.chartOption1 == 0) {
                chart
                .interval()
                .position('fprodname*fqty')
                .color('fprodname')
                .label('fqty', function(val){return {content: val + '件', offsetY: val == 0 ? -20 : 0}});
              } else {
                chart
                .interval()
                .position('fprodname*famount')
                .color('fprodname')
                .label('famount', function(val){
                  if (val >= 1000) {
                    val = parseFloat((val / 1000).toFixed(2)) + 'K';
                  }
                  return {content: '￥' + val, offsetY: val == 0 ? -20 : 0};
                });
              }

              chart.render();
            } else {
              //$ionicPopup.alert({
              //  title: res.data.msg
              //});
            }
          },
          function(err) {
          });
        }

        $scope.queryProductOfOrders();

        // 客单值分析
        $scope.hideChart4 = false;
        (userService.getRole() == '企业' ? qiyeShujuService : fenxiService).queryAmountOfOrders()
        .then(function(res) {
          if (res.data.code === 1 && res.data.data.length == 0) {
            $scope.hideChart4 = true;
          } else if (res.data.code === 1 && res.data.data.length > 0) {
            $scope.tips4 = res.data.tips;
            const data = res.data.data;
            const chart = new G2.Chart({
              container: 'xiaoshoufenxixitong-shouye-chart-4',
              autoFit: true,
              height: 300,
            });
            chartArray.push(chart)

            chart.data(data);
            chart.scale({
              fmonth: {
                // range: [0.1, 0.9],
                formatter: function(val) {
                  if (val.length >= 2) {
                    return Number(val.slice(-2)) + '月';
                  } else {
                    return val;
                  }
                }
              },
              famount: {
                range: [0, 0.9],
                formatter: function(val){
                  if (val >= 1000) {
                    val = parseFloat((val / 1000).toFixed(2)) + 'K';
                  }
                  return '￥' + val;
                },
              },
            });

            chart.tooltip({
              showTitle: false,
            });

            chart.line().position('fmonth*famount')
            .label('famount', function(val){
              if (val >= 1000) {
                val = parseFloat((val / 1000).toFixed(2)) + 'K';
              }
              return {content: '￥' + val};
            })
            .tooltip('fmonth*famount', function(fmonth, famount) {
              if (famount >= 1000) {
                famount = parseFloat((famount / 1000).toFixed(2)) + 'K';
              } else {
                famount = parseFloat(famount.toFixed(2));
              }
              return {
                name: fmonth + ' 客单值', value: '￥' + famount
              }
            });
            chart.point().position('fmonth*famount')
            .tooltip('fmonth*famount', function(fmonth, famount) {
              if (famount >= 1000) {
                famount = parseFloat((famount / 1000).toFixed(2)) + 'K';
              } else {
                famount = parseFloat(famount.toFixed(2));
              }
              return {
                name: fmonth + ' 客单值', value: '￥' + famount
              }
            });

            chart.render();
          } else {
            //$ionicPopup.alert({
            // title: res.data.msg
            // });
            $scope.hideChart4 = true;
          }
        },
        function(err) {
          $scope.hideChart4 = true;
        });

        // 小区占比分析
        $scope.hideChart5 = false;
        (userService.getRole() == '企业' ? qiyeShujuService : fenxiService).queryCommunityOfOrders()
        .then(function(res) {
          if (res.data.code === 1 && res.data.data.length == 0) {
            $scope.hideChart5 = true;
          } else if (res.data.code === 1 && res.data.data.length > 0) {
            $scope.tips5 = res.data.tips;
            var data = [];

            var sum = 0;
            res.data.data.forEach(function(v) {
              if (v.fcount != 0) {
                sum += v.fcount;
                data.push(v);
              }
            });

            const G = G2.getEngine('canvas');

            if (sum == 0) {
              $scope.hideChart5 = true;
            }
            let chart = new G2.Chart({
              container: 'xiaoshoufenxixitong-shouye-chart-5',
              autoFit: true,
              height: 400,
              localRefresh: false,
            });
            chartArray.push(chart)

            chart.data(data);
            chart.scale({
              fcount: {
                range: [0, 0.8],
                formatter: function(val) {
                  if (Number.isInteger(val)) {
                    return val + '单';
                  } else  {
                    return '';
                  }
                },
              }
            });
            chart.coordinate().transpose();

            chart.tooltip({
              showMarkers: false
            });
            chart.interval().position('fcommunity*fcount')
            .label('fcount', function(fcount){return {content: fcount + '单'}})
            .tooltip('fcommunity*fcount', function(fclass, fcount) {
              return {
                name: '数量', value: fcount + '单'
              };
            });

            chart.interaction('active-region');
            chart.legend({flipPage: false});
            chart.render();
          } else {
            //$ionicPopup.alert({
            // title: res.data.msg
            // });
            $scope.hideChart5 = true;
          }
        },
        function(err) {
          $scope.hideChart5 = true;
        });

        // 退单率分析
        $scope.hideChart6 = false;
        (userService.getRole() == '企业' ? qiyeShujuService : fenxiService).queryReturnCountOfOrders()
        .then(function(res) {
          if (res.data.code === 1 && res.data.data.length == 0) {
            $scope.hideChart6 = true;
          } else if (res.data.code === 1 && res.data.data.length > 0) {
            $scope.tips6 = res.data.tips;
            const data = res.data.data;
            const chart = new G2.Chart({
              container: 'xiaoshoufenxixitong-shouye-chart-6',
              autoFit: true,
              height: 300,
            });
            chartArray.push(chart)

            data.forEach(function(v){
              v.freturnrate *= 100;
            });

            chart.data(data);
            chart.scale({
              fmonth: {
                // range: [0.1, 0.9],
                formatter: function(val) {
                  if (val.length >= 2) {
                    return Number(val.slice(-2)) + '月';
                  } else {
                    return val;
                  }
                }
              },
              freturnrate: {
                range: [0, 0.9],
                formatter: function(val){
                  return parseFloat(val.toFixed(2)) + '%';
                },
              },
            });

            chart.tooltip({
              showTitle: false,
              showMarkers: false,
              itemTpl:
                '<li style="margin-bottom:4px;list-style-type:none;padding: 0;">' +
                '<span style="background-color:{color};" class="g2-tooltip-marker"></span>' +
                '{name}<br/>' +
                '<span style="padding-left: 16px;line-height: 16px;">退单率：{value}</span><br/>' +
                '<span style="padding-left: 16px;line-height: 16px;">退单数：{freturncount}</span><br/>' +
                '</li>',
            });
            chart.interaction('active-region');
            chart.legend(false);

            let colorMaps = [
              '#FF99A6', '#FF6F75', '#FF2641', '#D0021B', '#8E0909', '#6B0004'
            ];

            chart
            .interval()
            .position('fmonth*freturnrate')
            .color('fmonth')
            .label('freturnrate', function(val){return {content: parseFloat(val.toFixed(2)) + '%', offsetY: val == 0 ? -20 : 0}})
            .tooltip('fmonth*freturnrate*freturncount', function(fmonth, freturnrate,freturncount) {
              return {
                name: fmonth, 
                value: `${parseFloat(freturnrate.toFixed(2))}%`,
                freturncount
              }
            });

            chart.render();
          } else {
            //$ionicPopup.alert({
            // title: res.data.msg
            // });
            $scope.hideChart6 = true;
          }
        },
        function(err) {
          $scope.hideChart6 = true;
        });

        // 销售数据
        $scope.saleDataList = [];
        $scope.totalAmount = 0;
        (userService.getRole() == '企业' ? qiyeShujuService : fenxiService).queryCustomerOfOrders(1, 10, descending)
        .then(function(res) {
          if (res.data.code === 1) {
            $scope.tips7 = res.data.tips;
            $scope.saleDataList = res.data.data;
            $scope.saleDataList.forEach(saleData => {
              $scope.totalAmount += saleData.forderamount;
            });
          } else {
            //$ionicPopup.alert({
            // title: res.data.msg
            // });
          }

          $scope.hideLoading();
        },
        function(err) {
          $scope.hideLoading();
        });

        // 接待次数与首单成交分析
        $scope.hideChart7 = false;
        (userService.getRole() == '企业' ? qiyeShujuService : fenxiService).queryReceptionTimesOfFirstOrder()
        .then(function(res) {
          if (res.data.code === 1 && res.data.data.length == 0) {
            $scope.hideChart7 = true;
          } else if (res.data.code === 1 && res.data.data.length > 0) {
            $scope.tips8 = res.data.tips;
            var data = [];

            var sum = 0;
            res.data.data.forEach(function(v) {
              if (v.fcount != 0) {
                sum += v.fcount;
                data.push(v);
              }
            });

            if (sum == 0) {
              $scope.hideChart7 = true;
            }
            let chart = new G2.Chart({
              container: 'xiaoshoufenxixitong-shouye-chart-7',
              autoFit: true,
              height: 300,
            });
            chartArray.push(chart)
            chart.data(data);
            chart.scale('fcount', {
              formatter: function(val) {
                return val + '单';
              },
            });
            chart.coordinate('theta', {
              radius: 0.7
            });

            chart
              .interval()
              .adjust('stack')
              .position('fcount')
              .color('fclass')
              .label('fcount', {
                layout: [{ type: 'pie-spider' }, { type: 'hide-overlap' }],
                offset: 8,
                content: function(obj) {
                  const G = G2.getEngine('canvas');
                  const group = new G.Group({});
                  const [y0, y1] = [0, 0];
                  const inRight = y0 < y1;
                  const textAlign = inRight ? 'left' : 'right';
                  const topFontSize = 12;
                  group.addShape({
                    type: 'text',
                    attrs: {
                      x: 0,
                      y: 0,
                      text: `${obj.fclass}\n${obj.fcount}单\n${parseFloat((obj.fcount / sum * 100).toFixed(2))}%`,
                      fill: 'rgba(0, 0, 0, 0.65)',
                      fontSize: topFontSize,
                      textAlign,
                    },
                  });
                  if (!inRight) {
                    group.translate(group.getBBox().width, 0);
                  }
                  group.translate(0, topFontSize);
                  return group;
                },
                labelLine: {
                  style: {
                    lineWidth: 0.5,
                  },
                },
              })

            chart.interaction('element-active');
            chart.legend({flipPage: false});
            chart.render();
          } else {
            //$ionicPopup.alert({
            // title: res.data.msg
            // });
            $scope.hideChart7 = true;
          }
        },
        function(err) {
          $scope.hideChart7 = true;
        });

        // 小区客单值分析
        if (userService.getRole() == '经销商') {
          $scope.communityDepthOfOrdersList = [];
          $scope.totalAmount = 0;
          (fenxiService).queryCommunityDepthOfOrders()
          .then(function(res) {
            if (res.data.code === 1) {
              $scope.tips9 = res.data.tips;
              $scope.communityDepthOfOrdersList = res.data.data;
            } else {
              //$ionicPopup.alert({
              //  title: res.data.msg
              //});
            }

            $scope.hideLoading();
          },
          function(err) {
            $scope.hideLoading();
          });
        }

        $scope.setFavButtonInvisible = () => {
          $timeout.cancel($scope.timer);
          $scope.timer = $timeout(function() {
            $scope.show1 = false;
            $scope.show2 = false;
            $scope.show3 = false;
            $scope.show4 = false;
            $scope.show5 = false;
            $scope.show6 = false;
            $scope.show7 = false;
            $scope.show8 = false;
          }, 1500);
        }
      }, 500);

      $(function() {
        $('ion-content').scroll(function(ev) {
          if ($('div.shuju-topmenu-panel').length) {
            if ($('ion-content').scrollTop() >= 20 ||
                (ev.target.scrollTop && ev.target.scrollTop >= 20) ||
                (ev.originalEvent.detail && ev.originalEvent.detail.scrollTop >= 20)) {
              $('div.tabs-top div.shuju-topmenu-panel').addClass('fixed-header');
              $('ion-content.xiaoshoufenxixitong').css("top", "107px");
              $('.shuju-topmenu-panel').css('height', '63px');
              $('ul:not(.wheel-scroll)').removeClass('display-flex').removeClass('flex-wrap-wrap').addClass('scrolling-wrapper');
            } else {
              $('div.tabs-top div.shuju-topmenu-panel').removeClass('fixed-header');
              $('ion-content.xiaoshoufenxixitong').css("top", "233px");
              $('.shuju-topmenu-panel').css('height', '189px');
              $('ul:not(.wheel-scroll)').removeClass('scrolling-wrapper').addClass('display-flex').addClass('flex-wrap-wrap');
            }
          }
        });
      });

      $('ion-content.xiaoshoufenxixitong').css("top", "233px");
      $('.shuju-topmenu-panel').css('height', '189px');

      $scope.selectedItem = 1;

      $scope.showTip1 = false;
      $scope.showTip2 = false;
      $scope.showTip3 = false;
      $scope.showTip4 = false;
      $scope.showTip5 = false;
      $scope.showTip6 = false;
      $scope.showTip7 = false;
      $scope.showTip8 = false;
      $scope.showTip9 = false;

      $scope.toggleTip1 = function() {
        $scope.showTip1 = !$scope.showTip1;
      }

      $scope.toggleTip2 = function() {
        $scope.showTip2 = !$scope.showTip2;
      }

      $scope.toggleTip3 = function() {
        $scope.showTip3 = !$scope.showTip3;
      }

      $scope.toggleTip4 = function() {
        $scope.showTip4 = !$scope.showTip4;
      }

      $scope.toggleTip5 = function() {
        $scope.showTip5 = !$scope.showTip5;
      }

      $scope.toggleTip6 = function() {
        $scope.showTip6 = !$scope.showTip6;
      }

      $scope.toggleTip7 = function() {
        $scope.showTip7 = !$scope.showTip7;
      }

      $scope.toggleTip8 = function() {
        $scope.showTip8 = !$scope.showTip8;
      }

      $scope.toggleTip9 = function() {
        $scope.showTip9 = !$scope.showTip9;
      }

    });
})();

