(function(){
	"use strict";

	angular.module('starter.controllers')
    .controller('XiaoshouDongzuoFenxiController', function($rootScope, $scope, $ionicPopup, userService, fenxiService, qiyeShujuService) {
      $scope.showHeader();
      $scope.hideTabBar();

      $scope.initFenxiPeriod();

      const chart = new G2.Chart({
        container: 'xiaoshoufenxixitong-xiaoshoudongzuofenxi-chart',
        autoFit: true,
        height: 300,
        padding: [0, 100, 70],
      });
      $scope.$on('$destroy', function() {
        chart.clear()
        if (!chart.destroyed)
          chart.destroy()
      });
      
      $scope.loadData = () => {
        $scope.showLoading();

        if (!$scope.fenxiPeriodStart) {
          $scope.setFenxiPeriodStart(new Date());
          $scope.setFenxiPeriodEnd(new Date());
          $scope.setFenxiPeriodString($scope.fenxiPeriodStart.getFullYear() + '年' + ("0" + ($scope.fenxiPeriodStart.getMonth() + 1)).slice(-2) + '月');
        }

        // Draw charts
        $scope.hideChart1 = false;
        (userService.getRole() == '企业' ? qiyeShujuService : fenxiService).queryCustomerAction($scope.fenxiPeriodStart, $scope.fenxiPeriodEnd)
        .then(function(res) {
          if (res.data.code === 1 && res.data.data.length == 0) {
            $scope.hideChart1 = true;
            $scope.tips = ''
          } else if (res.data.code === 1 && res.data.data.length > 0) {
            $scope.tips = isRangeDate($scope.fenxiPeriodStart, $scope.fenxiPeriodEnd) ? res.data.tips : '';
            // Draw charts
            const dv = new DataSet.DataView().source(res.data.data);
            var sum = 0;
            for (var ii = 0; ii < res.data.data.length; ii++){
              if (res.data.data[ii].fcustaction == '进店') {
                sum = res.data.data[ii].fcount;
              }
            }
            if (sum == 0) {
              $scope.hideChart1 = true;
            }
            dv.transform({
              type: 'map',
              callback(row) {
                row.percent = parseFloat((row.fcount / sum * 100).toFixed(2));
                return row;
              },
            });
            const data = dv.rows;

            chart.data(data);
            chart.axis(false);
            chart.tooltip({
              showTitle: false,
              showMarkers: false,
              itemTpl:
                '<li style="margin-bottom:4px;list-style-type:none;padding: 0;">' +
                '<span style="background-color:{color};" class="g2-tooltip-marker"></span>' +
                '{name}<br/>' +
                '<span style="padding-left: 16px;line-height: 16px;">浏览人数：{fcount}</span><br/>' +
                '<span style="padding-left: 16px;line-height: 16px;">占比：{percent}</span><br/>' +
                '</li>',
            });
            chart
              .coordinate('rect')
              .transpose()
              .scale(1, -1);
            chart
              .interval()
              .adjust('symmetric')
              .position('fcustaction*percent')
              .style({
                fillOpacity: 1,
                opacity: 1,
              })
              .shape('funnel')
              .color('fcustaction')
              .label(
                'fcustaction*fcount',
                (fcustaction, fcount) => {
                  return {
                    content: `${fcustaction} ${fcount}`,
                  };
                },
                {
                  offset: 15,
                  labelLine: {
                    style: {
                      lineWidth: 1,
                      stroke: 'rgba(0, 0, 0, 0.15)',
                    },
                  },
                }
              )
              .tooltip('fcustaction*fcount*percent', (fcustaction, fcount, percent) => {
                return {
                  name: fcustaction,
                  percent: +percent + '%',
                  fcount,
                };
              })
              .animate({
                appear: {
                  animation: 'fade-in'
                },
                update: {
                  annotation: 'fade-in'
                }
              });

            chart.legend({flipPage: false});
            chart.interaction('element-active');
            let pp = 100 / res.data.data.length

            chart.on('beforepaint', () => {
              chart.annotation().clear(true);
              const chartData = chart.getData();
              // 中间标签文本
              chartData.forEach((obj, idx) => {
                let positionY = idx * pp   + pp / 2
                chart.annotation().text({
                  top: true,
                  position: ['50%', `${positionY}%`],
                  content: obj.percent + '%', // 显示的文本内容
                  style: {
                    stroke: null,
                    fill: '#fff',
                    fontSize: 12,
                    textAlign: 'center',
                    shadowBlur: 2,
                    shadowColor: 'rgba(51, 51, 51, .45)',
                  },
                });
              });

              chart.annotation().text({
                top: true,
                content: '* 以进店数作为统计基数',
                position: ['50%', '100%'],
                style: {
                  fill: '#666666',
                  fontSize: 12,
                  fontWeight: 300,
                  textAlign: 'center',
                },
                offsetY: 25,
              });
            });

            chart.render();
          } else {
            //$ionicPopup.alert({
            // title: res.data.msg
            // });
            $scope.hideChart1 = true;
          }

          $scope.hideLoading();
        },
        function(err) {
          $scope.hideChart1 = true;
          $scope.hideLoading();
        });
      }

      $scope.loadData();

      $scope.$on('refresh', function(ev) {
        $scope.loadData();
      });

      $scope.itemOnLongPress = function() {
        $scope.downloadImage(chart);
      }

    });
})();

